﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.IO;

namespace Niwar.Framework.Common
{
    public class NiwarRequest
    {
        /// <summary>
        /// 取得get值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static string get(string objName)
        {
            return Utils.ChkSQL(HttpContext.Current.Request[objName]);
        }
        /// <summary>
        /// 取得get值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static string get(string objName,string defaultValue)
        {
            if (HttpContext.Current.Request[objName] == null)
                return defaultValue;

            return Utils.ChkSQL(HttpContext.Current.Request[objName]);
        }
        /// <summary>
        /// 取得get值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static int getInt(string objName)
        {
            return Utils.StrToInt(HttpContext.Current.Request[objName], 0);
        }
        /// <summary>
        /// 取得get值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static int getInt(string objName, int defaultValue)
        {
            return Utils.StrToInt(HttpContext.Current.Request[objName], defaultValue);
        }
        /// <summary>
        /// 取得get值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static decimal getDecimal(string objName)
        {
            return Utils.StrToDecimal(HttpContext.Current.Request[objName], 0);
        }
        /// <summary>
        /// 取得多个值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static string[] getArray(string objName)
        {
            if (HttpContext.Current.Request[objName] == null) return null;
            if (string.IsNullOrEmpty(get(objName))) return new string[] { };
            return get(objName).Split(',');
        }
        /// <summary>
        /// 取得bool值
        /// </summary>
        /// <param name="objName"></param>
        /// <returns></returns>
        public static bool getBoolean(string objName) {
            if (HttpContext.Current.Request[objName] == null) return false;
            return get(objName).ToLower() == "true" || get(objName).ToLower() == "1" ? true : false;
        }
        /// <summary>
        /// 判断当前页面是否接收到了Post请求
        /// </summary>
        public static bool IsPost {
            get {
                return HttpContext.Current.Request.HttpMethod.Equals("POST");
            }
        }
        /// <summary>
        /// 判断当前页面是否接收到了Get请求
        /// </summary>
        public static bool IsGet
        {
            get
            {
                return HttpContext.Current.Request.HttpMethod.Equals("GET");
            }
        }
        /// <summary>
        /// 获得当前页面客户端的IP
        /// </summary>
        /// <returns>当前页面客户端的IP</returns>
        public static string GetIP()
        {
            string result = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"];
            if (string.IsNullOrEmpty(result))
                result = HttpContext.Current.Request.ServerVariables["HTTP_X_FORWARDED_FOR"];

            if (string.IsNullOrEmpty(result))
                result = HttpContext.Current.Request.UserHostAddress;

            if (string.IsNullOrEmpty(result) || !Utils.IsIP(result))
                return "127.0.0.1";

            return result;
        }
        /// <summary>
        /// 获得当前页面的名称
        /// </summary>
        /// <returns>当前页面的名称</returns>
        public static string GetPageName()
        {
            string[] urlArr = HttpContext.Current.Request.Url.AbsolutePath.Split('/');
            return urlArr[urlArr.Length - 1].ToLower();
        }

        /// <summary>
        /// 返回表单或Url参数的总个数
        /// </summary>
        /// <returns></returns>
        public static int GetParamCount()
        {
            return HttpContext.Current.Request.Form.Count + HttpContext.Current.Request.QueryString.Count;
        }

        /// <summary>
        /// 取得post过来的json对象并转换成字符串
        /// </summary>
        /// <param name="inputStream"></param>
        /// <returns></returns>
        public static string GetJson(Stream inputStream)
        {
            if (inputStream.Length > 0)
            {
                StreamReader streamReader = new StreamReader(inputStream);
                return streamReader.ReadToEnd();
            }
            return null;
        }
    }
}
