﻿using System.Collections.Generic;
using System.Web;
using System.Text;
using System.Linq;
using Niwar.Framework.ViewModel;

namespace Niwar.Framework.Data.PetaPoco
{
    public class PetaPocoRepository<T> where T : new()
    {
        public string ConnectionStringName { get; set; }

        protected Database db
        {
            get
            {
                if (HttpContext.Current.Items["NiwarDb"] == null)
                {
                    var retval = new Database(ConnectionStringName);
                    HttpContext.Current.Items["NiwarDb"] = retval;
                    return retval;
                }
                return (Database)HttpContext.Current.Items["NiwarDb"];
            }
        } 

        /// <summary>
        /// 获取单条记录：根据主键
        /// </summary>
        /// <param name="primaryKey"></param>
        /// <returns></returns>
        public T Single(object primaryKey)
        {
            return db.SingleOrDefault<T>(primaryKey);
        }
        /// <summary>
        /// 获取单条记录：根据Sql
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public T Single(Sql sql)
        {
            return db.SingleOrDefault<T>(sql);
        }
        /// <summary>
        /// 是否存在：根据主键
        /// </summary>
        /// <param name="primaryKey"></param>
        /// <returns></returns>
        public bool Exists(object primaryKey) {
            return db.Exists<T>(primaryKey);
        }
        /// <summary>
        /// 是否存在：根据Sql
        /// </summary>
        /// <param name="sqlCondition"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public bool Exists(string sqlCondition, params object[] args)
        {
            return db.Exists<T>(sqlCondition, args);
        }
        /// <summary>
        /// 查询列表：根据Sql
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public IEnumerable<T> Query(Sql sql)
        {
            return db.Query<T>(sql);
        }
        /// <summary>
        /// 查询列表：根据Sql
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public IEnumerable<T> Query(string sql,params object[] args)
        {
            return db.Query<T>(sql,args);
        }
        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="sql"></param>
        /// <returns></returns>
        public Page<T> Paged(long page,long pageSize,Sql sql)
        {
            return db.Page<T>(page, pageSize, sql);
        }

        /// <summary>
        /// extjs分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="resultCount"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="sql"></param>
        /// <returns></returns>
        public List<T> Paged(long page, long pageSize, out long resultCount, List<string> selectFileds = null, List<DataFilter> filters = null, List<DataSort> sorts = null, Sql sql = null)
        {
            Page<T> p = db.Page<T>(page, pageSize, selectFileds, filters, sorts, sql);
            resultCount = p.TotalItems;

            return p.Items;
        }

        /// <summary>
        /// 根据指定条件获取所有数据
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="db"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="sql"></param>
        /// <returns></returns>
        public List<T> GetAll(List<string> selectFileds = null, List<DataFilter> filters = null, List<DataSort> sorts = null, Sql sql = null)
        {
            return db.GetAll<T>(selectFileds, filters, sorts, sql);
        }
        
        /// <summary>
        /// 新增或更新
        /// </summary>
        /// <param name="entity"></param>
        public void SaveOrUpdate(T entity)
        {
            db.Save(entity, false);
        }
        /// <summary>
        /// 批量新增或更新
        /// </summary>
        /// <param name="entitys"></param>
        public void SaveOrUpdate(List<T> entitys)
        {
            using (var trans = db.GetTransaction())
            {
                entitys.ForEach(delegate(T entity) {
                    db.Save(entity,false);
                });
                trans.Complete();
            }
        }
        /// <summary>
        /// 插入
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public object Insert(T entity)
        {
            return db.Insert(entity);
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="entity"></param>
        public int Update(T entity)
        {
            return db.Update(entity);
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public int Update(string sql,params object[] args)
        {
            return db.Update<T>(sql,args);
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="entity"></param>
        public void Delete(T entity)
        {
            db.Delete(entity);
        }
        /// <summary>
        /// 根据条件删除
        /// </summary>
        /// <param name="sql"></param>
        public void Delete(Sql sql)
        {
            db.Delete<T>(sql);
        }
        /// <summary>
        /// 根据主键删除
        /// </summary>
        /// <param name="Id"></param>
        public void DeleteById(object Id)
        {
            db.Delete<T>(Id);
        }
        /// <summary>
        /// 执行sql
        /// </summary>
        /// <param name="sql"></param>
        public void Execute(Sql sql)
        {
            db.Execute(sql);
        }
    }
}