﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nimmo.Framework.Common;
using Nimmo.Framework.Data.PetaPoco;
using Nimmo.Framework.ViewModel;

namespace Nimmo.Infrastructure.Repositories
{
    public class LogRepository : SingleRepository<OperationLog>
    {
        /// <summary>
        /// extjs分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="resultCount"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <returns></returns>
        public List<OperationLog> Paged(int UserID, string logType, long page, long pageSize, out long resultCount, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            Sql sql = Sql.Builder;
            sql.Append(string.Format("select {0} from OperationLogs", selectFileds == null ? "*" : String.Join(",", selectFileds)));
            sql.Where("LogType=@0", logType);
            if (UserID > 0)
                sql.Where("UserID=@0", UserID);

            return this.Paged(page, pageSize, out resultCount, selectFileds, filters, sorts, sql);
        }

        /// <summary>
        /// 获取所有数据
        /// </summary>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="selectFileds"></param>
        /// <returns></returns>
        public List<OperationLog> GetAll(int UserID, string logType, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            Sql sql = Sql.Builder;
            sql.Append(string.Format("select {0} from OperationLogs", selectFileds == null ? "*" : String.Join(",", selectFileds)));
            sql.Where("LogType=@0", logType);
            if (UserID > 0)
                sql.Where("UserID=@0", UserID);

            return this.GetAll(selectFileds, filters, sorts, sql);
        }

        /// <summary>
        /// 根据用户ID取该用户的历史登录数据
        /// </summary>
        /// <param name="UserID"></param>
        /// <returns></returns>
        public IList<OperationLog> GetListByUserID(int UserID)
        {
            Sql sql = Sql.Builder.Where("UserID=@0", UserID).OrderBy("AddTime Desc");

            return this.Query(sql).ToList();
        }

        /// <summary>
        /// 清空错误日志
        /// </summary>
        public void ClearErrorLogs()
        {
            this.Execute(Sql.Builder.Append("truncate table ErrorLogs"));
        }
    }
}
