﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nimmo.Framework.Common;
using Nimmo.Framework.Data.PetaPoco;
using Nimmo.Framework.ViewModel;

namespace Nimmo.Infrastructure.Repositories
{
    public class RoleActionRepository : SingleRepository<RoleAction>
    {
        /// <summary>
        /// 获取指定用户拥有的菜单信息
        /// </summary>
        /// <param name="UserID"></param>
        /// <returns></returns>
        public IList<RoleAction> GetRoleActionsByUserID(int UserID)
        {
            Sql sql = Sql.Builder.Select("RoleActions.*").From("RoleActions")
                            .LeftJoin("RolesToActions").On("RolesToActions.RoleActionId=RoleActions.RoleActionId")
                            .LeftJoin("Roles").On("Roles.RoleId=RolesToActions.RoleId")
                            .LeftJoin("RolesToUsers").On("RolesToUsers.RoleId=Roles.RoleId")
                            .LeftJoin("Users").On("Users.UserId=RolesToUsers.UserId")
                            .Where("Users.UserId=@0", UserID);

            return this.Query(sql).ToList();
        }

        /// <summary>
        /// 根据角色ID获取该角色拥有的权限列表
        /// </summary>
        /// <param name="RoleID"></param>
        /// <returns></returns>
        public IList<RoleAction> GetListByRoleID(int RoleID)
        {
            Sql sql = Sql.Builder.Select("*").From("RoleActions")
                .LeftJoin("RolesToActions").On("RolesToActions.RoleActionId=RoleActions.RoleActionId")
                .Where("RolesToActions.RoleId=@0", RoleID);

            return this.Query(sql).ToList();
        }

        /// <summary>
        /// 获取所有权限
        /// </summary>
        /// <returns></returns>
        public IList<RoleAction> GetAll()
        {
            return this.Query(Sql.Builder.Where("1=1")).ToList();
        }
    }
}
