﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nimmo.Framework.Common;
using Nimmo.Framework.Data.PetaPoco;
using Nimmo.Infrastructure.Relators;

namespace Nimmo.Infrastructure.Repositories
{
    public class RoleRepository : SingleRepository<Role>
    {
        /// <summary>
        /// 取得所有角色
        /// </summary>
        /// <returns></returns>
        public IList<Role> GetAll()
        {
            Sql sql = Sql.Builder.Where("1=1");

            return this.Query(sql).ToList();
        }

        /// <summary>
        /// 获取映射后的角色
        /// </summary>
        /// <param name="RoleID"></param>
        /// <returns></returns>
        public Role GetMappedById(int RoleID)
        {
            Sql sql = Sql.Builder.Select("Roles.*", "RoleMenus.*").From("Roles")
                .LeftJoin("RolesToMenus").On("RolesToMenus.RoleID=Roles.RoleID")
                .LeftJoin("RoleMenus").On("RoleMenus.RoleMenuId=RolesToMenus.RoleMenuId")
                .Where("Roles.RoleID=@0", RoleID);

            return this.db.Fetch<Role, RoleMenu, Role>(
                new RoleRelator().Map, sql).Single();
        }

        /// <summary>
        /// 保存菜单拥有的角色
        /// </summary>
        /// <param name="MenuId">菜单ID</param>
        /// <param name="RoleId">角色ID</param>
        /// <param name="IsDelete">是否为删除</param>
        public void SaveRoleMenu(int MenuId, int RoleId, bool IsDelete)
        {
            Sql sql = Sql.Builder.Append("insert into RolesToMenus (RoleMenuId,RoleId) values (@0,@1)", MenuId, RoleId);
            if (IsDelete)
                sql = Sql.Builder.Append("delete from RolesToMenus where RoleMenuId=@0 and RoleId=@1",MenuId,RoleId);

            this.Execute(sql);
        }

        /// <summary>
        /// 保存菜单拥有的权限
        /// </summary>
        /// <param name="ActionId"></param>
        /// <param name="RoleId"></param>
        /// <param name="IsDelete"></param>
        public void SaveRoleAction(int ActionId, int RoleId, bool IsDelete)
        {
            Sql sql = Sql.Builder;

            if (IsDelete)
                sql = Sql.Builder.Append("delete from RolesToActions where RoleActionId=@0 and RoleId=@1", ActionId, RoleId);
            else
            {
                sql = sql.Select("COUNT(*)").From("RolesToActions").Where("RoleActionId=@0 and RoleId=@1", ActionId, RoleId);
                if (this.db.Query<long>(sql).Single() == 0)
                    sql = Sql.Builder.Append("insert into RolesToActions (RoleActionId,RoleId) values (@0,@1)", ActionId, RoleId);
            }

            this.Execute(sql);
        }

        /// <summary>
        /// 删除角色
        /// </summary>
        /// <param name="RoleId"></param>
        public void Delete(int RoleId)
        {
            using (var trans = this.db.GetTransaction())
            {
                Sql sql = Sql.Builder.Append("delete from RolesToMenus where RoleId=@0", RoleId);
                //删除菜单的关联
                this.Execute(sql);
                //删除权限的关联
                sql = Sql.Builder.Append("delete from RolesToActions where RoleId=@0", RoleId);
                this.Execute(sql);
                //删除用户的关联
                sql = Sql.Builder.Append("delete from RolesToUsers where RoleId=@0", RoleId);
                this.Execute(sql);
                //删除角色
                this.DeleteById(RoleId);

                trans.Complete();
            }
        }
    }
}
