﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nimmo.Framework.Common;
using Nimmo.Framework.ViewModel;
using Nimmo.Infrastructure.Repositories;

namespace Nimmo.Infrastructure.ViewModel
{
    public class LogModel
    {
        private readonly LogRepository _LogRepository = new LogRepository();

        #region 静态实例
        private static volatile LogModel instance = null;
        private static object lockHelper = new object();

        public static LogModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new LogModel();
                        }
                    }
                }

                return instance;
            }
        } 
        #endregion

        /// <summary>
        /// extjs分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="resultCount"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <returns></returns>
        public List<OperationLog> Paged(int UserID,string logType, long page, long pageSize, out long resultCount, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            return _LogRepository.Paged(UserID,logType, page, pageSize, out resultCount, filters, sorts, selectFileds);
        }

        /// <summary>
        /// 获取所有数据
        /// </summary>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="selectFileds"></param>
        /// <returns></returns>
        public List<OperationLog> GetAll(int UserID, string logType, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            return _LogRepository.GetAll(UserID, logType, filters, sorts, selectFileds);
        }

        /// <summary>
        /// 添加操作日志
        /// </summary>
        /// <param name="UserID"></param>
        public void AddOperationLog(int UserID, string Message, string logType)
        {
            _LogRepository.SaveOrUpdate(new OperationLog()
            {
                IPAddress = Utils.GetIP(),
                AddTime = DateTime.Now,
                UserId = UserID,
                LogType = logType,
                Message = Message
            });
        }

        /// <summary>
        /// 清空错误日志
        /// </summary>
        public void ClearErrorLogs()
        {
            _LogRepository.ClearErrorLogs();
        }
    }
}
