﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Nimmo.Infrastructure.Repositories;
using Nimmo.Framework.ViewModel;

namespace Nimmo.Infrastructure.ViewModel
{
    public class RoleActionModel
    {
        /// <summary>
        /// 权限名称
        /// </summary>
        public string ActionName { get; set; }

        /// <summary>
        /// 对应Url
        /// </summary>
        public string ActionUrl { get; set; }

        /// <summary>
        /// 权限描述
        /// </summary>
        public string ActionDesc { get; set; }

        #region 静态实例
        private static volatile RoleActionModel instance = null;
        private static object lockHelper = new object();

        public static RoleActionModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new RoleActionModel();
                        }
                    }
                }

                return instance;
            }
        }
        #endregion

        private RoleActionRepository _RoleActionRepository = new RoleActionRepository();

        /// <summary>
        /// extjs分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="resultCount"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <returns></returns>
        public List<RoleAction> Paged(long page, long pageSize, out long resultCount, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds=null)
        {
            return _RoleActionRepository.Paged(page, pageSize, out resultCount, selectFileds, filters, sorts);
        }

        /// <summary>
        /// 获取所有数据
        /// </summary>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="selectFileds"></param>
        /// <returns></returns>
        public List<RoleAction> GetAll(List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            return _RoleActionRepository.GetAll(selectFileds, filters, sorts);
        }

        /// <summary>
        /// 根据角色ID获取该角色拥有的权限列表
        /// </summary>
        /// <param name="RoleID"></param>
        /// <returns></returns>
        public IList<RoleAction> GetListByRoleID(int RoleID)
        {
            return _RoleActionRepository.GetListByRoleID(RoleID);
        }

        /// <summary>
        /// 获取所有权限
        /// </summary>
        /// <returns></returns>
        public IList<RoleAction> GetAll()
        {
            return _RoleActionRepository.GetAll();
        }

        /// <summary>
        /// 获取单个对象
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public RoleAction Single(int id)
        {
            return _RoleActionRepository.Single(id);
        }

        /// <summary>
        /// 保存单个角色
        /// </summary>
        /// <param name="entity"></param>
        public void SaveOrUpdate(RoleAction entity)
        {
            _RoleActionRepository.SaveOrUpdate(entity);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="RoleId"></param>
        public void Delete(int RoleActionId)
        {
            _RoleActionRepository.DeleteById(RoleActionId);
        }
    }
}
