﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nimmo.Framework;
using Nimmo.Framework.ViewModel;
using Nimmo.Infrastructure.Repositories;

namespace Nimmo.Infrastructure.ViewModel
{
    public class RoleMenuModel
    {
        /// <summary>
        /// 上级ID
        /// </summary>
        public int ParentID { get; set; }

        /// <summary>
        /// 菜单名称
        /// </summary>
        public string MenuName { get; set; }

        /// <summary>
        /// 菜单链接
        /// </summary>
        public string MenuLink { get; set; }

        /// <summary>
        /// 菜单参数
        /// </summary>
        public string MenuConfig { get; set; }

        /// <summary>
        /// 有哪些功能按钮
        /// </summary>
        public List<string> MenuButtons { get; set; }

        /// <summary>
        /// 控制器名称
        /// </summary>
        public string ControllerName { get; set; }

        private readonly RoleMenuRepository _RoleMenuRepository = new RoleMenuRepository();

        #region 静态实例
        private static volatile RoleMenuModel instance = null;
        private static object lockHelper = new object();

        public static RoleMenuModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new RoleMenuModel();
                        }
                    }
                }

                return instance;
            }
        }
        #endregion

        /// <summary>
        /// 获取所有菜单
        /// </summary>
        /// <returns></returns>
        public IList<RoleMenu> GetAll()
        {
            return _RoleMenuRepository.GetAll();
        }

        /// <summary>
        /// 获取树形菜单数据
        /// </summary>
        /// <param name="role"></param>
        /// <returns></returns>
        public IList<MenuTree> GetMenuTree(Role role = null)
        {
            return this.GetMenuTree(this.GetAll(), role);
        }

        /// <summary>
        /// 获取单个对象
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public RoleMenu Single(int id)
        {
            return _RoleMenuRepository.Single(id);
        }

        /// <summary>
        /// 保存多个对象
        /// </summary>
        /// <param name="entitys"></param>
        public void SaveOrUpdate(List<RoleMenu> entitys)
        {
            _RoleMenuRepository.SaveOrUpdate(entitys);
        }

        /// <summary>
        /// 保存单个对象
        /// </summary>
        /// <param name="entity"></param>
        public void SaveOrUpdate(RoleMenu entity)
        {
            _RoleMenuRepository.SaveOrUpdate(entity);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="ID"></param>
        public void Delete(int ID)
        {
            var entity = _RoleMenuRepository.Single(ID);
            if (entity != null)
            {
                if (_RoleMenuRepository.Query("WHERE ParentID=@0",ID).Count() > 0)
                    throw new NimmoException(string.Format("名称为[{0}]的菜单有下级菜单，请先删除下级。",entity.MenuName));

                _RoleMenuRepository.Delete(ID);
            }
        }

        /// <summary>
        /// 获取树形菜单数据
        /// </summary>
        /// <param name="menus"></param>
        /// <returns></returns>
        public IList<MenuTree> GetMenuTree(IList<RoleMenu> menus, Role role = null)
        {
            var result = new List<MenuTree>();
            Recursion(new MenuTree { id = 0, children = new List<MenuTree>() }, result, menus, role);
            return result;
        }

        /// <summary>
        /// 树形菜单
        /// </summary>
        public class MenuTree : ExtTreeData<MenuTree>
        {
            public Int32 SortOrder { get; set; }
            public string ControllerName { get; set; }
            public string Buttons { get; set; }
            public bool expanded { get; set; }
            public string url { get; set; }
            public bool? @checked { get; set; }
        }
        //递归
        private void Recursion(MenuTree parentNode, List<MenuTree> result, IList<RoleMenu> list, Role role = null)
        {
            var childs = list.Where(p => p.ParentID == parentNode.id).OrderBy(p => p.SortOrder);
            foreach (var item in childs)
            {
                var childCount = list.Where(p => p.ParentID == item.Id).Count();
                var child = new MenuTree
                {
                    id = item.Id,
                    SortOrder = item.SortOrder,
                    text = item.MenuName,
                    ControllerName=item.ControllerName,
                    Buttons=item.MenuButtons,
                    expanded = true,
                    leaf = childCount == 0,
                    url = item.MenuLink,
                    children = new List<MenuTree>()
                };
                if (role != null)
                {
                    child.@checked =role.Menus.Where(p => p.Id == item.Id).Count() > 0 ? true: false;
                }
                if (item.ParentID <= 0)
                    result.Add(child);
                else
                    parentNode.children.Add(child);

                Recursion(child, result, list, role);
            }
        }
    }
}
