﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Nimmo.Infrastructure.Repositories;
using Nimmo.Framework.ViewModel;

namespace Nimmo.Infrastructure.ViewModel
{
    public class RoleModel
    {
        /// <summary>
        /// 角色代码
        /// </summary>
        public string RoleCode { get; set; }

        /// <summary>
        /// 角色名称
        /// </summary>
        public string RoleName { get; set; }

        private readonly RoleRepository _RoleRepository = new RoleRepository();

        #region 静态实例
        private static volatile RoleModel instance = null;
        private static object lockHelper = new object();

        public static RoleModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new RoleModel();
                        }
                    }
                }

                return instance;
            }
        }
        #endregion

        /// <summary>
        /// 获取所有角色列表
        /// </summary>
        /// <returns></returns>
        public IList<Role> GetAll()
        {
            return _RoleRepository.GetAll();
        }

        /// <summary>
        /// 获取所有数据
        /// </summary>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="selectFileds"></param>
        /// <returns></returns>
        public List<Role> GetAll(List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            return _RoleRepository.GetAll(selectFileds, filters, sorts);
        }

        /// <summary>
        /// 保存多个角色
        /// </summary>
        /// <param name="entitys"></param>
        public void SaveOrUpdate(List<Role> entitys)
        {
            _RoleRepository.SaveOrUpdate(entitys);
        }

        /// <summary>
        /// 保存单个角色
        /// </summary>
        /// <param name="entity"></param>
        public void SaveOrUpdate(Role entity)
        {
            _RoleRepository.SaveOrUpdate(entity);
        }

        /// <summary>
        /// 获取单个对象
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public Role Single(int id)
        {
            return _RoleRepository.Single(id);
        }

        /// <summary>
        /// 获取映射后的角色
        /// </summary>
        /// <param name="RoleID"></param>
        /// <returns></returns>
        public Role GetMappedById(int RoleID)
        {
            return _RoleRepository.GetMappedById(RoleID);
        }

        /// <summary>
        /// 保存菜单拥有的角色
        /// </summary>
        /// <param name="MenuId">菜单ID</param>
        /// <param name="RoleId">角色ID</param>
        /// <param name="IsDelete">是否为删除</param>
        public void SaveRoleMenu(int MenuId, int RoleId, bool IsDelete)
        {
            _RoleRepository.SaveRoleMenu(MenuId, RoleId, IsDelete);
        }

        /// <summary>
        /// 保存菜单拥有的权限
        /// </summary>
        /// <param name="ActionId"></param>
        /// <param name="RoleId"></param>
        /// <param name="IsDelete"></param>
        public void SaveRoleAction(int ActionId, int RoleId, bool IsDelete)
        {
            _RoleRepository.SaveRoleAction(ActionId, RoleId, IsDelete);
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="RoleId"></param>
        public void Delete(int RoleId)
        {
            _RoleRepository.Delete(RoleId);
        }
    }
}
