﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;
using Nimmo.Infrastructure.Repositories;

namespace Nimmo.Infrastructure.ViewModel
{
    public class TeamModel
    {
        private readonly TeamRepository _TeamRepository = new TeamRepository();

        /// <summary>
        /// 团队名称
        /// </summary>
        [Required]
        public string TeamName { get; set; }

        /// <summary>
        /// 上级团队
        /// </summary>
        public int ParentID { get; set; }

        /// <summary>
        /// 团队描述
        /// </summary>
        public string Description { get; set; }

        #region 静态实例
        private static volatile TeamModel instance = null;
        private static object lockHelper = new object();

        public static TeamModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new TeamModel();
                        }
                    }
                }

                return instance;
            }
        }
        #endregion

        /// <summary>
        /// 获取所有团队数据
        /// </summary>
        /// <returns></returns>
        public IList<Team> GetAll()
        {
            return _TeamRepository.GetAll();
        }

        /// <summary>
        /// 根据主键ID获取单个对象
        /// </summary>
        /// <param name="TeamID"></param>
        /// <returns></returns>
        public Team GetById(int TeamID)
        {
            return _TeamRepository.GetById(TeamID);
        }

        /// <summary>
        /// 保存多个团队
        /// </summary>
        /// <param name="entitys"></param>
        public void SaveOrUpdate(List<Team> entitys)
        {
            _TeamRepository.SaveOrUpdate(entitys);
        }

        /// <summary>
        /// 保存单个团队
        /// </summary>
        /// <param name="entity"></param>
        public void SaveOrUpdate(Team entity)
        {
            _TeamRepository.SaveOrUpdate(entity);
        }

        /// <summary>
        /// 删除团队
        /// </summary>
        /// <param name="TeamID"></param>
        public void Delete(int TeamID)
        {
            _TeamRepository.Delete(TeamID);
        }
    }
}
