﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;

using Nimmo.Framework.Common;
using Nimmo.Framework.ViewModel;
using Nimmo.Infrastructure.Repositories;

namespace Nimmo.Infrastructure.ViewModel
{
    public class UserModel
    {
        private readonly UserInfoRepository _UserInfoRepository = new UserInfoRepository();

        /// <summary>
        /// 登录名
        /// </summary>
        public string LoginName { get; set; }

        /// <summary>
        /// 密码
        /// </summary>
        public string Pwd { get; set; }

        /// <summary>
        /// 确认密码
        /// </summary>
        public string ConfirmLoginPwd { get; set; }

        /// <summary>
        /// 新密码
        /// </summary>
        public string NewPwd { get; set; }

        /// <summary>
        /// 姓名
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 邮箱
        /// </summary>
        public string Email { get; set; }

        #region 静态实例
        private static volatile UserModel instance = null;
        private static object lockHelper = new object();

        public static UserModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new UserModel();
                        }
                    }
                }

                return instance;
            }
        }
        #endregion

        /// <summary>
        /// extjs分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="resultCount"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <returns></returns>
        public List<UserInfo> Paged(int TeamID, long page, long pageSize, out long resultCount, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            return _UserInfoRepository.Paged(TeamID, page, pageSize, out resultCount, filters, sorts, selectFileds);
        }

        /// <summary>
        /// 获取所有数据
        /// </summary>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="selectFileds"></param>
        /// <returns></returns>
        public List<UserInfo> GetAll(List<DataFilter> filters = null, List<DataSort> sorts = null, List<string> selectFileds = null)
        {
            return _UserInfoRepository.GetAll(selectFileds, filters, sorts);
        }

        /// <summary>
        /// 获取新注册用户
        /// </summary>
        /// <param name="dateType">时间类型：m本月、w本周、d今天</param>
        /// <returns></returns>
        public List<UserInfo> GetNewUsers(string dateType)
        {
            return _UserInfoRepository.GetNewUsers(dateType);
        }

        /// <summary>
        /// 密码输入是否正确
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="Pwd"></param>
        /// <returns></returns>
        public bool ExistUsersByPwd(int UserID)
        {
            return _UserInfoRepository.ExistUsersByPwd(UserID, this.Pwd);
        }

        /// <summary>
        /// 修改密码
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="Pwd"></param>
        public void UpdatePwd(int UserID)
        {
            _UserInfoRepository.UpdatePwd(UserID, this.Pwd);
        }

        /// <summary>
        /// 根据用户ID获取用户信息
        /// </summary>
        /// <param name="UserID"></param>
        /// <returns></returns>
        public UserInfo GetByUserID(int UserID)
        {
            return _UserInfoRepository.GetByUserID(UserID);
        }

        /// <summary>
        /// 保存用户信息
        /// </summary>
        /// <param name="entity"></param>
        public void SaveOrUpdate(UserInfo entity)  
        {
            _UserInfoRepository.SaveOrUpdateUserInfo(entity);
        }
    }
}
