﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Nimmo.Infrastructure.Repositories;
using Nimmo.Framework.ViewModel;

namespace Nimmo.Infrastructure.ViewModel
{
    public class UserPortalModel
    {
        private readonly UserPortalRepository _UserPortalRepository = new UserPortalRepository();

        /// <summary>
        /// 页面代码
        /// </summary>
        public string PageCode { get; set; }

        /// <summary>
        /// 布局json
        /// </summary>
        public string PortalJson { get; set; }

        #region 静态实例
        private static volatile UserPortalModel instance = null;
        private static object lockHelper = new object();

        public static UserPortalModel GetInstance
        {
            get
            {
                if (instance == null)
                {
                    lock (lockHelper)
                    {
                        if (instance == null)
                        {
                            instance = new UserPortalModel();
                        }
                    }
                }

                return instance;
            }
        } 
        #endregion

        /// <summary>
        /// 获取指定用户的所有组件状态
        /// </summary>
        /// <param name="UserID"></param>
        /// <returns></returns>
        public List<UserPortalModel> GetAll(int UserID)
        {
            var data = _UserPortalRepository.GetAll(UserID);
            List<UserPortalModel> result = new List<UserPortalModel>();
            data.ForEach(delegate(UserPortal up)
            {
                result.Add(new UserPortalModel() { PageCode = up.PageTag, PortalJson = up.JosnPortal });
            });

            return result;
        }

        /// <summary>
        /// 获取当前用户指定页面的布局信息
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="PageCode"></param>
        /// <returns></returns>
        public string GetPortal(int UserID)
        {
            return _UserPortalRepository.GetPortal(UserID, this.PageCode);
        }

        /// <summary>
        /// 保存指定用户指定页面的布局信息
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="PageCode"></param>
        /// <param name="PortalJson"></param>
        public void SavePortal(int UserID)
        {
            _UserPortalRepository.SavePortal(UserID, this.PageCode, this.PortalJson);
        }
    }
}
