﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using Niwar.Framework;

namespace Niwar.MVC.Filter
{
    public class AdminAuthorizeFilterAttribute : ActionFilterAttribute
    {
        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            if (filterContext == null)
            {
                throw new ArgumentNullException("filterContext");
            }

            object[] naAttrs = filterContext.ActionDescriptor.GetCustomAttributes(typeof(NoAuthorizeAttribute), true);
            var isNoAuthorize = naAttrs.Length == 1;//当前Action是否不需要验证
            if (isNoAuthorize) return;

            object[] vpAttrs = filterContext.ActionDescriptor.GetCustomAttributes(typeof(ViewPageAttribute), true);
            var isViewPage = vpAttrs.Length == 1;//当前Action请求是否为具体的功能页

            string msg = "";
            if (this.AuthorizeCore(filterContext, isViewPage,out msg) == false)//根据验证判断进行处理
            {
                if (isViewPage == true)
                {
                    filterContext.Result = new RedirectResult("/Home/Login");//直接URL输入的页面地址跳转到登陆页
                }
                else
                {
                    filterContext.Result = new ContentResult { Content = @"Niwar.infoTip('"+ msg +"')" };//功能权限弹出提示框
                }
            }
        }
        //权限判断业务逻辑
        protected virtual bool AuthorizeCore(ActionExecutingContext filterContext, bool isViewPage,out string msg)
        {
            msg = "未登录，访问非法";
            if (filterContext.HttpContext == null)
            {
                throw new ArgumentNullException("httpContext");
            }

            //判定用户是否登录
            if (!HttpContext.Current.User.Identity.IsAuthenticated)
                return false;

            var controllerName = filterContext.RouteData.Values["controller"].ToString();
            var actionName = filterContext.RouteData.Values["action"].ToString();
            
            return true;
        }
    }
}
